--[[
	Modified version of Telo's original QuickLoot by Ascendant
	Date: 08.05.05
	
	With these functions I have provided the mod with the ability to be configured
	Allow for confguration to be saved between sessions per char per realm
	Configuration options:
		turn autohide on and off
		enable or disable movement of loot window  to mouse pointer for each item in loot box
		allow user to enable/disable the mod
	
]]
	

function QuickLoot_OnEvent(event)

	if (event == "VARIABLES_LOADED") then
		QL_VarLoaded = true;
		if (not QL_ModLoaded) then
			QL_ThisChar = QL_GetPlayerAndRealmNames();
			if ((QL_ThisChar ~= nil) and (QL_ThisChar ~= UNKNOWNOBJECT) and (QL_ThisChar ~= "unknown")) then
				QL_Init();
			end
		end
		return;
	end
	
	if ((event == "UNIT_NAME_UPDATE") or (event == "PLAYER_ENTERING_WORLD")) then
		if (not QL_ModLoaded) then
			QL_ThisChar = QL_GetPlayerAndRealmNames();
			if ((QL_ThisChar ~= nil) and (QL_ThisChar ~= UNKNOWNOBJECT) and (QL_ThisChar ~= "unknown")) then
				QL_Init();
			end
		end
		return;
	end

end

function QL_GetPlayerAndRealmNames() 
	-- Finds and returns Char name and Realm name to calling function
	
	local realmName = GetCVar("realmName");
	local playerName = UnitName("player");
	
	if ((playerName ~= nil) and (playerName ~= UNKNOWNOBJECT) and (playerName ~= UKNOWNBEING)) then
		return realmName.."-"..playerName;		
	end
	
	return "unknown";
end

function QuickLoot_SlashCmdHndlr(msg)
	-- slash command handler
	
	msg = string.lower(msg);	
	
	if ((not msg) or (msg == nil) or (strlen(msg) <= 0) or (strfind(msg, "help"))) then
	
		DEFAULT_CHAT_FRAME:AddMessage("QuickLoot help information:");
		DEFAULT_CHAT_FRAME:AddMessage("\"/quickloot move\" toggles whether or not to move the loot window to the mouse pointer for each item in the loot box.  (on by default)");
		DEFAULT_CHAT_FRAME:AddMessage("\"/quickloot autohide\" toggle whether or not to automatically hide the loot window if there is no loot to collect.  (on by default)");
		DEFAULT_CHAT_FRAME:AddMessage("\"/quickloot enable\" enables QuickLoot.  (on by default)");
		DEFAULT_CHAT_FRAME:AddMessage("\"/quickloot disable\" disables QuickLoot.");
		DEFAULT_CHAT_FRAME:AddMessage("\"/quickloot reset\" resets QuickLoot to its default functionality.");
		DEFAULT_CHAT_FRAME:AddMessage("\"/quickloot default\" displays QuickLoot's default options.");
		DEFAULT_CHAT_FRAME:AddMessage("\"/quickloot display\" displays the current settings for QuickLoot.");
		
	elseif (strfind(msg, "move")) then
		-- determines if the loot window will move to the mouse pointer for each item in the loot box - loot box will always move to mouse pointer when loot box is opened
		
		if (QuickLoot_Config[QL_ThisChar].Enable) then
		
			if (QuickLoot_Config[QL_ThisChar].DontMove) then
			
				QuickLoot_Config[QL_ThisChar].DontMove = false;
				DEFAULT_CHAT_FRAME:AddMessage("Loot frame will now move to mouse pointer for each item in the Loot Box.");
			
			else
			
				QuickLoot_Config[QL_ThisChar].DontMove = true;
				DEFAULT_CHAT_FRAME:AddMessage("Loot frame will no longer move for each item in the Loot Box.");
				
			end
		
		else
		
			DEFAULT_CHAT_FRAME:AddMessage("QuickLoot must be enabled to change its settings.");
			
		end
		
	elseif (strfind(msg, "autohide")) then
		-- determines if the loot window should autohide when nothing is in the loot box
		
		if (QuickLoot_Config[QL_ThisChar].Enable) then
		
			if (QuickLoot_Config[QL_ThisChar].AutoHide) then
			
				QuickLoot_Config[QL_ThisChar].AutoHide = false;
				DEFAULT_CHAT_FRAME:AddMessage("AutoHide has been disabled.");
				
			else
			
				QuickLoot_Config[QL_ThisChar].AutoHide = true;
				DEFAULT_CHAT_FRAME:AddMessage("AutoHide has been enabled.");
				
			end
			
		else
		
			DEFAULT_CHAT_FRAME:AddMessage("QuickLoot must be enabled to change its settings.");
			
		end
		
	elseif (strfind(msg, "reset")) then
		-- reset to defaults
		
		if (QuickLoot_Config[QL_ThisChar].Enable) then
		
			QuickLoot_Config[QL_ThisChar].DontMove = false;
			QuickLoot_Config[QL_ThisChar].AutoHide = true;
			DEFAULT_CHAT_FRAME:AddMessage("QuickLoot settings have been reset to default.");
			
		else
			
			DEFAULT_CHAT_FRAME:AddMessage("You must enable QuickLoot to reset its settings to default.");
			
		end
		
	elseif (strfind(msg, "enable")) then
		-- enable mod ingame
		
		if (QuickLoot_Config[QL_ThisChar].Enable) then
		
			DEFAULT_CHAT_FRAME:AddMessage("QuickLoot is already enabled.");
			
		else
		
			QuickLoot_Config[QL_ThisChar].Enable = true;
			DEFAULT_CHAT_FRAME:AddMessage("QuickLoot has been enabled.");
			
		end
		
	elseif (strfind(msg, "disable")) then
		-- disable mod ingame
		
		if (not QuickLoot_Config[QL_ThisChar].Enable) then
			
			DEFAULT_CHAT_FRAME:AddMessage("QuickLoot is already disabled.");
			
		else
		
			QuickLoot_Config[QL_ThisChar].Enable = false;
			DEFAULT_CHAT_FRAME:AddMessage("QuickLoot has been disabled.");
			
		end
			
		
	elseif (strfind(msg, "default")) then
		-- displays default values for QuickLoot
		
		DEFAULT_CHAT_FRAME:AddMessage("Default settings for QuickLoot:");
		DEFAULT_CHAT_FRAME:AddMessage("QuickLoot is enabled by default to move the loot box to the mouse pointer for each item in the loot box.");
		DEFAULT_CHAT_FRAME:AddMessage("QuickLoot is enabled by default to automatically hide the loot box if there is nothing in it.");
		
	elseif (strfind(msg, "display")) then
		-- displays current settings for QuickLoot
		
		if (QuickLoot_Config[QL_ThisChar].Enable) then
		
			DEFAULT_CHAT_FRAME:AddMessage("Current QuickLoot settings:");
		
			if (QuickLoot_Config[QL_ThisChar].DontMove) then
				DEFAULT_CHAT_FRAME:AddMessage("The loot window will NOT move for each item in the loot box.");
			else
				DEFAULT_CHAT_FRAME:AddMessage("The loot window WILL move for each item in the loot box.");
			end
			
			if (QuickLoot_Config[QL_ThisChar].AutoHide) then
				DEFAULT_CHAT_FRAME:AddMessage("The loot window WILL automatically hide if there is no loot to collect in the loot box.");
			else
				DEFAULT_CHAT_FRAME:AddMessage("The loot window will NOT automatically hide if there is no loot to collect in the loot box.");
			end
			
		else
		
			DEFAULT_CHAT_FRAME:AddMessage("You must enable QuickLoot to view its current settings.");
			
		end
		
	end
	
end

function QL_InitVars()
	-- initialize variables
	
	if (QuickLoot_Config[QL_ThisChar].DontMove == nil) then
		QuickLoot_Config[QL_ThisChar].DontMove = false;
	end
	if (QuickLoot_Config[QL_ThisChar].AutoHide == nil) then
		QuickLoot_Config[QL_ThisChar].AutoHide = true;
	end
	if (QuickLoot_Config[QL_ThisChar].Enable == nil) then
		QuickLoot_Config[QL_ThisChar].Enable = true;
	end
		
end

function QL_CheckVars()
	-- Checks to make sure that the char has all correct variables, if not, initialize varaible with default values
	
	local bVarChange = false;
	
	if (QuickLoot_Config[QL_ThisChar].DontMove == nil) then
		QuickLoot_Config[QL_ThisChar].DontMove = false;
		bVarChange = true;
	end
	if (QuickLoot_Config[QL_ThisChar].AutoHide == nil) then
		QuickLoot_Config[QL_ThisChar].AutoHide = true;
		bVarChange = true;
	end
	if (QuickLoot_Config[QL_ThisChar].Enable == nil) then
		QuickLoot_Config[QL_ThisChar].Enable = true;
		bVarChange = true;
	end
	
	if (bVarChange) then
	
		DEFAULT_CHAT_FRAME:AddMessage("QuickLoot variables have been updated.");
		
	end

end

function QL_Init()
	-- initialize mod for first use
	
	if (QL_ModLoaded == true) then	-- should not even be here
	return;
	end
	if (QL_VarLoaded == false) then	-- ensures that the variable have been loaded properly before continiuing
		return;
	end
	if ((not QuickLoot_Config)) then	-- if 'config' variable has not been created then create it
		QuickLoot_Config = {};		
	end
	if (not QuickLoot_Config[QL_ThisChar]) then	-- if 'config' info for current char has not been created then create it
		QuickLoot_Config[QL_ThisChar] = {};
		QL_InitVars();
	else
		QL_CheckVars();		-- if 'config' info has been created then check to make sure that all vars have been
	end
	DEFAULT_CHAT_FRAME:AddMessage("QuickLoot setting have been loaded for "..QL_ThisChar);
	QL_ModLoaded = true;

end